'use client';
import { useState, useEffect } from 'react';
import { useAuth, useTVMode } from '@/components/context';
import type { UserWithStatus } from '@/lib/hooks';
import { BasedAvatar, Drawer, DrawerTrigger, Loading } from '@/components/ui';
import { StatusMessage, SubmitButton } from '@/components/default';
import { ConnectionStatus, HistoryDrawer } from '@/components/status';
import type { Profile } from '@/utils/supabase';
import { makeConditionalClassName } from '@/lib/utils';
import { Card, CardContent, CardHeader } from '@/components/ui/card';
import { Checkbox } from '@/components/ui/checkbox';
import { Input } from '@/components/ui/input';
import { Button } from '@/components/ui/button';
import { RefreshCw, Clock, Calendar } from 'lucide-react';
import { useStatusData, useSharedStatusSubscription } from '@/lib/hooks';
import { formatTime, formatDate } from '@/lib/utils';
import Link from 'next/link';

type ListProps = {
  initialStatuses: UserWithStatus[]
};

export const StatusList = ({ initialStatuses = [] }: ListProps) => {
  const { isAuthenticated } = useAuth();
  const { tvMode } = useTVMode();

  const [selectedUsers, setSelectedUsers] = useState<UserWithStatus[]>([]);
  const [selectAll, setSelectAll] = useState(false);
  const [statusInput, setStatusInput] = useState('');
  const [selectedHistoryUser, setSelectedHistoryUser] =
    useState<Profile | null>(null);
  const [updateStatusMessage, setUpdateStatusMessage] = useState('');

  const {
    data: usersWithStatuses = initialStatuses,
    isLoading: loading,
    error,
    refetch,
    newStatuses,
    updateStatusMutation,
  } = useStatusData({
    initialData: initialStatuses,
    enabled: isAuthenticated,
  });

  // In your StatusList component
  const { connectionStatus, connect: reconnect } = useSharedStatusSubscription(() => {
    refetch().catch((error) => {
      console.error('Error refetching statuses:', error);
    });
  });

  //const { connectionStatus, connect: reconnect } = useStatusSubscription({
    //enabled: isAuthenticated,
    //onStatusUpdate: () => {
      //refetch().catch((error) => {
        //console.error('Error refetching statuses:', error);
      //});
    //},
  //});

  const handleUpdateStatus = () => {
    if (!isAuthenticated) {
      setUpdateStatusMessage(
        'Error: You must be signed in to update technician statuses!'
      );
      return;
    }

    if (statusInput.length < 3 || statusInput.length > 80) {
      setUpdateStatusMessage(
        'Error: Your status must be between 3 & 80 characters long!'
      );
      return;
    }

    updateStatusMutation.mutate({
      usersWithStatuses: selectedUsers,
      status: statusInput.trim(),
    });

    setSelectedUsers([]);
    setStatusInput('');
    setUpdateStatusMessage('');
  };

  const handleCheckboxChange = (user: UserWithStatus) => {
    setSelectedUsers((prev) =>
      prev.some((u) => u.user.id === user.user.id)
        ? prev.filter((prevUser) => prevUser.user.id !== user.user.id)
        : [...prev, user]
    );
  };

  const handleSelectAllChange = () => {
    if (selectAll) {
      setSelectedUsers([]);
    } else {
      setSelectedUsers(usersWithStatuses);
    }
    setSelectAll(!selectAll);
  };

  useEffect(() => {
    setSelectAll(
      selectedUsers.length === usersWithStatuses.length &&
        usersWithStatuses.length > 0
    );
  }, [selectedUsers.length, usersWithStatuses.length]);

  if (loading) {
    return (
      <div className='flex justify-center items-center min-h-[400px]'>
        <Loading className='w-full' alpha={0.5} />
      </div>
    );
  }

  if (error) {
    return (
      <div className='flex flex-col justify-center items-center min-h-[400px] gap-4'>
        <p className='text-red-500'>Error loading status updates</p>
        <Button onClick={() => refetch()} variant='outline'>
          <RefreshCw className='w-4 h-4 mr-2' />
          Retry
        </Button>
      </div>
    );
  }

  const containerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'flex flex-col mx-auto space-y-4 items-center',
    on: 'lg:w-11/12 w-full mt-15',
    off: 'sm:w-5/6 md:3/4 lg:w-1/2',
  });

  const headerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'w-full',
    on: 'hidden',
    off: 'flex mb-4 justify-between',
  });

  const cardContainerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: '',
    on: '',
    off: 'space-y-3 items-center justify-center w-full',
  });

  const cardClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName:
      'transition-all duration-300 hover:shadow-md hover:bg-muted/50 cursor-pointer',
    on: 'lg:text-4xl',
    off: 'lg:text-base lg:w-full',
  });

  return (
    <div className={containerClassName}>
      <div className={headerClassName}>
        <div className='flex items-center gap-10'>
          <div className='flex gap-2'>
            <Checkbox
              id='select-all'
              checked={selectAll}
              onCheckedChange={handleSelectAllChange}
              className='size-6'
            />
            <label htmlFor='select-all' className='font-medium'>
              Select All
            </label>
          </div>
          {!tvMode && (
            <div className='flex flex-row gap-2'>
              <p>Miss the old table?</p>
              <Link
                href='/status/table'
                className='italic font-semibold text-accent-foreground'
              >
                Find it here!
              </Link>
            </div>
          )}
          </div>
        <div className='flex items-center gap-2'>
          <ConnectionStatus
            status={connectionStatus}
            onReconnect={reconnect}
            showAsButton={connectionStatus === 'disconnected'}
          />
        </div>
      </div>

      <div className={cardContainerClassName}>
        {usersWithStatuses.map((userWithStatus) => {
          const isSelected = selectedUsers.some(
            (u) => u.user.id === userWithStatus.user.id
          );
          const isNewStatus = newStatuses.has(userWithStatus);
          const isUpdatedByOther =
            userWithStatus.updated_by &&
            userWithStatus.updated_by.id !== userWithStatus.user.id;

          return (
            <Card
              key={userWithStatus.user.id}
              className={`
                ${cardClassName}
                ${isSelected ? 'ring-2 ring-primary' : ''}
                ${isNewStatus ? 'animate-pulse bg-primary/5 border-primary/20' : ''}
              `}
            >
              <CardHeader className='pb-3'>
                <div className='flex items-center justify-between'>
                  <div className='flex items-center gap-3'>
                    {!tvMode && (
                      <Checkbox
                        checked={isSelected}
                        onCheckedChange={() =>
                          handleCheckboxChange(userWithStatus)
                        }
                        onClick={(e) => e.stopPropagation()}
                      />
                    )}
                    <BasedAvatar
                      src={userWithStatus.user.avatar_url}
                      fullName={userWithStatus.user.full_name}
                      className={tvMode ? 'w-24 h-24' : 'w-16 h-16'}
                    />
                    <div className='my-auto'>
                      <h3
                        className={`font-semibold ${tvMode ? 'text-5xl' : 'text-2xl'}`}
                      >
                        {userWithStatus.user.full_name}
                      </h3>
                      {isUpdatedByOther && (
                        <div className='flex items-center gap-1 text-muted-foreground'>
                          <BasedAvatar
                            src={userWithStatus.updated_by?.avatar_url}
                            fullName={userWithStatus.updated_by?.full_name}
                            className='w-5 h-5'
                          />
                          <span
                            className={`${tvMode ? 'text-3xl' : 'text-sm'}`}
                          >
                            Updated by {userWithStatus.updated_by?.full_name}
                          </span>
                        </div>
                      )}
                    </div>
                  </div>
                  <div className='my-auto'>
                    <div className='flex items-center gap-2 text-muted-foreground'>
                      <Clock className={`${tvMode ? 'w-8 h-8' : 'w-6 h-6'}`} />
                      <span className={`${tvMode ? 'text-3xl' : 'text-xl'}`}>
                        {formatTime(userWithStatus.created_at)}
                      </span>
                    </div>
                    <div className='flex items-center gap-2 text-muted-foreground'>
                      <Calendar
                        className={`${tvMode ? 'w-8 h-8' : 'w-6 h-6'}`}
                      />
                      <span className={`${tvMode ? 'text-3xl' : 'text-xl'}`}>
                        {formatDate(userWithStatus.created_at)}
                      </span>
                    </div>
                  </div>
                </div>
              </CardHeader>
              <CardContent className='pt-0'>
                <Drawer>
                  <DrawerTrigger asChild>
                    <div
                      className={`
                        p-4 rounded-lg bg-muted/30 hover:bg-muted/50
                        transition-colors cursor-pointer text-left
                        ${tvMode ? 'text-4xl' : 'text-xl'}
                      `}
                      onClick={() =>
                        setSelectedHistoryUser(userWithStatus.user)
                      }
                    >
                      <p className='font-medium'>{userWithStatus.status}</p>
                    </div>
                  </DrawerTrigger>
                  {selectedHistoryUser === userWithStatus.user && (
                    <HistoryDrawer user={selectedHistoryUser} />
                  )}
                </Drawer>
              </CardContent>
            </Card>
          );
        })}
      </div>

      {usersWithStatuses.length === 0 && (
        <Card className='p-8 text-center'>
          <p
            className={`text-muted-foreground ${tvMode ? 'text-4xl' : 'text-lg'}`}
          >
            No status updates have been made in the past day.
          </p>
        </Card>
      )}

      {!tvMode && (
        <Card className='p-6 mt-6'>
          <div className='flex flex-col gap-4'>
            <h3 className='text-lg font-semibold'>Update Status</h3>
            <div className='flex flex-col gap-4'>
              <div className='flex gap-4'>
                <Input
                  autoFocus
                  type='text'
                  placeholder='Enter status'
                  className='flex-1 text-base'
                  value={statusInput}
                  disabled={updateStatusMutation.isPending}
                  onChange={(e) => setStatusInput(e.target.value)}
                  onKeyDown={(e) => {
                    if (
                      e.key === 'Enter' &&
                      !e.shiftKey &&
                      !updateStatusMutation.isPending
                    ) {
                      e.preventDefault();
                      handleUpdateStatus();
                    }
                  }}
                />
                <SubmitButton
                  onClick={handleUpdateStatus}
                  disabled={updateStatusMutation.isPending}
                  className='px-6'
                >
                  {selectedUsers.length > 0
                    ? `Update status for ${selectedUsers.length}
                      ${selectedUsers.length > 1 ? 'users' : 'user'}`
                    : 'Update status'
                  }
                </SubmitButton>
              </div>
              {updateStatusMessage &&
                (updateStatusMessage.includes('Error') ||
                updateStatusMessage.includes('error') ||
                updateStatusMessage.includes('failed') ||
                updateStatusMessage.includes('invalid') ? (
                  <StatusMessage message={{ error: updateStatusMessage }} />
                ) : (
                  <StatusMessage message={{ message: updateStatusMessage }} />
              ))}
            </div>
            <div className='flex justify-center mt-2'>
              <Drawer>
                <DrawerTrigger asChild>
                  <Button
                    variant='outline'
                    className={tvMode ? 'text-3xl p-6' : ''}
                  >
                    View All Status History
                  </Button>
                </DrawerTrigger>
                <HistoryDrawer />
              </Drawer>
            </div>
          </div>
        </Card>
      )}
    </div>
  );
};
